// some RDF helper functions

const GENERAL_NS = "http://www.xulplanet.com/rdf/reobase#";
const RP_NS      = "http://www.xulplanet.com/ns/rpath-template";
const RDF_NS     = "http://www.w3.org/1999/02/22-rdf-syntax-ns#";
const RDFS_NS    = "http://www.w3.org/2000/01/rdf-schema#";
const DC_NS      = "http://purl.org/dc/elements/1.1/";
const XUL_NS     = "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";

const DEFAULT_STORE_DSURI = "chrome://topicalla/content/ontology.rdf";
const FOAFSCHEMA_DSURI = "chrome://topicalla/content/foaf-schema.rdf";

var RDF="@mozilla.org/rdf/rdf-service;1";
RDF=Components.classes[RDF].getService(Components.interfaces.nsIRDFService);

var gLastDatasource = null;

function getProfileStore()
{
  var dserv=Components.classes["@mozilla.org/file/directory_service;1"]
              .getService(Components.interfaces.nsIProperties);

  var profileFile = dserv.get("ProfD",Components.interfaces.nsIFile).clone();
  profileFile.append("topicalla-store.rdf");

  var ioservice = Components.classes["@mozilla.org/network/io-service;1"]
                    .getService(Components.interfaces.nsIIOService);

  var storeURI = ioservice.newFileURI(profileFile);
  var storeURIStr = storeURI.spec;

  if (!profileFile.exists()){
    var dsource = RDF.GetDataSourceBlocking(DEFAULT_STORE_DSURI);
    if (dsource instanceof Components.interfaces.nsIRDFRemoteDataSource){
      dsource.FlushTo(storeURIStr);
    }
  }
  return RDF.GetDataSourceBlocking(storeURIStr);
}

function RDFAssert(ds,subj,prop,obj)
{
  if (!obj){
//    dump("No object in: " + subj.Value + " -> " + prop + " -> [NULL]\n");
    return;
  }
  if (typeof obj == "string") obj = RDF.GetLiteral(obj);

//  dump("Asserting: " + subj.Value + " -> " + prop + " -> " + RDFGetValue(obj) + "\n");
  ds.Assert(subj,RDF.GetResource(prop),obj,true);
}

function RDFAssertOnce(ds,subj,prop,obj)
{
  var propres = RDF.GetResource(prop);
  var oldobj = ds.GetTarget(subj,propres,true);

  if (typeof obj == "string") obj = RDF.GetLiteral(obj);

/*
  if (subj) dump("Asserting: " + subj.Value + " -> " + prop + " ->");
  else dump("Asserting: " + subj + " -> " + prop);
  if (oldobj) dump(" [" + RDFGetValue(oldobj) + "]");
  dump(" " + RDFGetValue(obj) + "\n");
*/

  if (oldobj != null){ ds.Change(subj,propres,oldobj,obj); }
  else ds.Assert(subj,propres,obj,true);
}
 
function RDFUnassert(ds,subj,prop,obj)
{
  prop=RDF.GetResource(prop);
  if (obj == null) obj=ds.GetTarget(subj,prop,true);
  if (obj != null) ds.Unassert(subj,prop,obj,true);
}

function RDFAssertAll(ds,subj,arr)
{
  for (var t=0; t<arr.length; t+=2){
    ds.Assert(subj,RDF.GetResource(arr[t]),RDF.GetLiteral(arr[t+1]),true);
  }
}

function RDFUnassertAll(ds,subj,removeChildren)
{
  if (typeof subj == "string") subj=RDF.GetResource(subj);
  var iter=ds.ArcLabelsOut(subj);
  while (iter.hasMoreElements()){
    var prop=iter.getNext();
    var obj=ds.GetTarget(subj,prop,true);
    ds.Unassert(subj,prop,obj,true);
  }

  if (removeChildren){
    var container=RDFGetContainer(ds,subj);
    if (container != null){
      var children=container.GetElements();
      while (children.hasMoreElements()){
        var child=children.getNext();
        RDFUnassertAll(ds,child,false);
      }
    }
  }
}

function RDFGetValue(node)
{
  return ((node instanceof Components.interfaces.nsIRDFResource) ? node.Value :
          ((node instanceof Components.interfaces.nsIRDFLiteral) ? node.Value : ""));
}

function RDFGetResource(ds,subj,prop)
{
  var obj=ds.GetTarget(subj,RDF.GetResource(prop),true);
  return ((obj instanceof Components.interfaces.nsIRDFResource) ? obj : null);
}

function RDFGetLiteral(ds,subj,prop)
{
  var obj=ds.GetTarget(subj,RDF.GetResource(prop),true);
  return ((obj instanceof Components.interfaces.nsIRDFLiteral) ? obj.Value : null);
}

function RDFGetContainer(ds,node)
{
  var RDFContainer = '@mozilla.org/rdf/container;1';
  RDFContainer = Components.classes[RDFContainer].createInstance(
                   Components.interfaces.nsIRDFContainer);
 
  try {
    RDFContainer.Init(ds,node);
    return RDFContainer;
  }
  catch (ex){
    return null;
  }
}

function createNewDatasource()
{
  return Components.classes["@mozilla.org/rdf/datasource;1?name=in-memory-datasource"]
                           .createInstance(Components.interfaces.nsIRDFDataSource);
}

function addDatasource(ds)
{
  if (!gGeneralDatasource){
    gGeneralDatasource=Components.classes["@mozilla.org/rdf/datasource;1?name=composite-datasource"]
                         .createInstance(Components.interfaces.nsIRDFCompositeDataSource);
  }
  gGeneralDatasource.AddDataSource(ds);
  gLastDatasource = ds;
}

function isDataSourceLoaded(dsuri)
{
  var iter = gGeneralDatasource.GetDataSources();
  while (iter.hasMoreElements()){
    var ds = iter.getNext().QueryInterface(Components.interfaces.nsIRDFDataSource);
    if (ds.URI == dsuri) return true;
  }
  return false;
}

function createRPathTemplateBuilder(ds,defns,listener)
{
  var builder=Components.classes["@xulplanet.com/rdf/rpath-templatebuilder;1"]
              .createInstance(Components.interfaces.rpIRPathTemplateBuilder);

  if (ds) builder.dataSource=ds;
  if (defns) builder.defaultNamespace=defns;
  if (listener) builder.listener=listener;
  return builder;
}

function evaluateRPath(ds,defns,context,expr)
{
  var evaluator=Components.classes["@xulplanet.com/rdf/rpath-evaluator;1"]
                .createInstance(Components.interfaces.rpIRPathEvaluator);

  if (ds) evaluator.dataSource=ds;
  if (defns) evaluator.defaultNamespace=defns;
  if (context) evaluator.setSingleContextNode(context);
  if (expr) evaluator.expression=expr;

  return evaluator.evaluate(1);
}

function rebuildPage(builder,elem,context)
{
  builder.parseTemplate(elem,context);
}

function serializeRDF(ds)
{
  var outputStream = {
    data: "",
    close : function(){},
    flush : function(){},
    write : function (buffer,count){
      this.data += buffer;
      return count;
    },
    writeFrom : function (stream,count){},
    isNonBlocking: false
  }
  serializeRDFToStream(ds,outputStream);
  return outputStream.data;
}

function serializeRDFToStream(ds,outputStream)
{
  var ser=Components.classes["@mozilla.org/rdf/xml-serializer;1"]
                    .createInstance(Components.interfaces.nsIRDFXMLSerializer);
  ser.init(ds);
  ser.QueryInterface(Components.interfaces.nsIRDFXMLSource).Serialize(outputStream);
}
